      REAL FUNCTION DIST(M, X, Y, N, P)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      COMPUTES THE EUCLIDEAN DISTANCE BETWEEN EITHER TWO CASES OR TWO
C      VARIABLES
C
C   DESCRIPTION
C   -----------
C
C   1.  THE PARAMETERS X AND Y ARE ASSUMED TO BE EITHER A VECTOR OR A
C       STARTING LOCATION IN A MATRIX.  IF THEY ARE VECTORS OR THE
C       DISTANCE BETWEEN TWO COLUMNS OF A MATRIX ARE REQUIRED, SET N=1.
C       IF THE DISTANCE BETWEEN TWO ROWS OF A MATRIX ARE REQUIRED, SET
C       N EQUAL TO THE LEADING DIMENSION OF THE MATRIX AS DECLARED IN
C       THE MAIN PROGRAM.
C
C   2.  P IS A PARAMETER DECIDING WHICH DISTANCE FUNCTION TO USE.  IF P
C       IS GREATER THAN ZERO, THE MINKOWSKI DISTANCE IS COMPUTED,
C       RAISING THE DIFFERENCES TO THE P-TH POWER.  OTHERWISE, THE
C       DISTANCE RETURNED WILL BE THE PROPORTION OF CORRESPONDING
C       ELEMENTS OF X AND Y WHOSE DIFFERENCE IS GREATER THAN THE
C       ABSOLUTE VALUE OF P.
C
C   3.  MISSING VALUES ARE DENOTED BY 99999.  AND ARE IGNORED BY THE
C       DISTANCE CALCULATIONS.
C
C   INPUT PARAMETERS
C   ----------------
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF ELEMENTS.
C
C   X     REAL VECTOR DIMENSIONED AT LEAST (M-1)*N+1 (UNCHANGED ON
C            OUTPUT).
C         THE STARTING LOCATION OF THE FIRST VECTOR OF ELEMENTS.
C
C   Y     REAL VECTOR DIMENSIONED AT LEAST (M-1)*N+1 (UNCHANGED ON
C            OUTPUT).
C         THE STARTING LOCATION OF THE SECOND VECTOR OF ELEMENTS.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE SKIP FACTOR BETWEEN ELEMENTS.
C
C         IF THE DISTANCE BETWEEN COLUMNS OF A MATRIX IS DESIRED
C            OR IF X AND Y ARE INDIVIDUAL VECTORS, SET N = 1.
C
C         IF THE DISTANCE BETWEEN ROWS OF A MATRIX IS DESIRED,
C            SET N = DIM, WHERE DIM IS THE LEADING DIMENSION OF THE
C            ARRAY THE ELEMENTS X AND Y COME FROM.
C
C   P     REAL SCALAR (UNCHANGED ON OUTPUT).
C         P .GT. 0 IMPLIES THE P-TH POWER DISTANCE.
C            I.E. SUM EACH DIFFERENCE RAISED TO THE P-TH POWER, THEN
C            RETURN THE P-TH ROOT OF THE SUM AS THE DISTANCE
C         P .LE. 0 RETURNS THE PROPORTION OF DISTANCES GREATER THAN THE
C            ABSOLUTE VALUE OF P.
C
C   OUTPUT PARAMETER
C   ----------------
C
C   DIST  REAL SCALAR.
C         THE DISTANCE BETWEEN THE TWO VECTORS.
C
C   REFERENCE
C   ---------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGE 58-65, 69.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      DIMENSION X(*),Y(*)
      DD=0.
      DP=0.
      DO 10 I=1,M
         II = (I-1) * N + 1
         IF(X(II).NE.99999..AND.Y(II).NE.99999.) THEN
            DP=DP+1
            DIF=ABS(X(II)-Y(II))
            IF(P.GT.0.) DD=DD+DIF**P
            IF(P.LE.0..AND.DIF+P.GT.0.) DD=DD+1
         ENDIF
   10 CONTINUE
      IF(DP.NE.0.) THEN
         IF(P.GT.0.) DD=(DD/DP)**(1./P)
         IF(P.LE.0.) DD=DD/DP
      ENDIF
      DIST=DD
      RETURN
      END
