      SUBROUTINE JOIN(MM, M, N, A, RLAB, TITLE, DMIWRK, IWORK, WORK,
     *                TLAB, IERR, OUNIT)
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
C   PURPOSE
C   -------
C
C      USES A GENERAL JOINING ALGORITHM TO FORM AND OUTPUT A TREE OF
C      CLUSTERS OF CASES
C
C   DESCRIPTION
C   -----------
C
C   1.  THE VARIABLES MUST BE SCALED SIMILARLY (CLUSTER SUBROUTINE
C       STAND CAN BE USED TO STANDARDIZE THE VARIABLES.)
C
C   2.  INITIALLY, EACH OBJECT IS ASSUMED TO BE A CLUSTER.  THEN THE
C       TWO CLOSEST CLUSTERS ARE AMALGAMATED TO FORM A NEW CLUSTER.
C       THIS PROCESS OF COMBINING THE CLOSEST TWO CLUSTERS CONTINUES
C       UNTIL ONLY ONE CLUSTER REMAINS.  A TREE IS MAINTAINED TO SHOW
C       ALL CLUSTERS AT EACH STEP.
C
C   3.  THE OUTPUT IS THE STANDARD TREE WRITTEN ON FORTRAN UNIT OUNIT.
C       IT LISTS THE CASES VERTICALLY AND HAS HORIZONTAL LINES
C       EMANATING FROM EACH CASE.  EACH CLUSTER WILL CORRESPOND TO A
C       VERTICAL LINE BETWEEN TWO HORIZONTAL LINES.  THE CASES BETWEEN
C       AND INCLUDED IN THE HORIZONTAL LINES ARE THE MEMBERS OF THE
C       CLUSTER.  THE DISTANCE FROM THE CASE NAMES TO THE VERTICAL
C       LINES CORRESPOND TO THE CLUSTER DIAMETER OR THE DISTANCE AT
C       WHICH THE TWO CLUSTERS WERE JOINED.
C
C   INPUT PARAMETERS
C   ----------------
C
C   MM    INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX A.  MUST BE AT LEAST M.
C
C   M     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF CASES.
C
C   N     INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE NUMBER OF VARIABLES.
C
C   A     REAL MATRIX WHOSE FIRST DIMENSION MUST BE MM AND WHOSE SECOND
C            DIMENSION MUST BE AT LEAST N (DESTROYED DURING EXECUTION).
C         THE MATRIX OF DISTANCES BETWEEN CASES.
C
C         A(I,J) IS THE VALUE FOR THE J-TH VARIABLE FOR THE I-TH CASE.
C
C   RLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST N.
C            (UNCHANGED ON OUTPUT).
C         THE LABELS OF THE CASES.
C
C   TITLE 10-CHARACTER VARIABLE (UNCHANGED ON OUTPUT).
C         TITLE OF THE DATA SET.
C
C   DMIWRK INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         THE FIRST DIMENSION OF THE MATRIX IWORK.  MUST BE AT LEAST 4.
C
C   IWORK INTEGER MATRIX WHOSE FIRST DIMENSION MUST BE DMIWRK AND SECOND
C            DIMENSION MUST BE AT LEAST M.
C         WORK MATRIX.
C
C   WORK  REAL VECTOR DIMENSIONED AT LEAST M.
C         WORK VECTOR.
C
C   TLAB  VECTOR OF 4-CHARACTER VARIABLES DIMENSIONED AT LEAST M+1.
C         WORK VECTOR.
C
C   OUNIT INTEGER SCALAR (UNCHANGED ON OUTPUT).
C         UNIT NUMBER FOR OUTPUT.
C
C   OUTPUT PARAMETER
C   ----------------
C
C   IERR  INTEGER SCALAR.
C         ERROR FLAG.
C
C         IERR = 0, NO ERRORS WERE DETECTED DURING EXECUTION
C
C         IERR = 1, EITHER THE FIRST AND LAST CASES OR THE CLUSTER
C                   DIAMETER FOR A CLUSTER IS OUT OF BOUNDS.  THE
C                   CLUSTER AND ITS VALUES ARE PRINTED ON UNIT OUNIT.
C                   EXECUTION WILL CONTINUE WITH QUESTIONABLE RESULTS
C                   FOR THAT CLUSTER.
C
C   REFERENCES
C   ----------
C
C     HARTIGAN, J. A. (1975).  CLUSTERING ALGORITHMS, JOHN WILEY &
C        SONS, INC., NEW YORK.  PAGES 216-232.
C
C     HARTIGAN, J. A. (1975) PRINTER GRAPHICS FOR CLUSTERING. JOURNAL OF
C        STATISTICAL COMPUTATION AND SIMULATION. VOLUME 4,PAGES 187-213.
C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
C
      INTEGER DMIWRK, OUNIT
      DIMENSION A(MM,*), IWORK(DMIWRK,*), WORK(*)
      CHARACTER*4 RLAB(*), TLAB(*)
      CHARACTER*10 TITLE
C
      IERR = 0
      DO 10 I=1,M
         TLAB(I) = RLAB(I)
   10    IWORK(3,I)=I
C
C     FIND CLOSEST PAIR
C
      DO 40 K=1,M-1
         DMIN=R1MACH(2)
         IMIN=2
         JMIN=3
         DO 30 I=1,M
            IF(IWORK(3,I).GE.0) THEN
               DO 20 J=I,M
                  IF(IWORK(3,J).GE.0.AND.J.NE.I) THEN
                     Z=DIST(N,A(I,1),A(J,1),MM,2.)
                     IF(Z.LT.DMIN) THEN
                        IMIN=I
                        JMIN=J
                        DMIN=Z
                     ENDIF
                  ENDIF
   20          CONTINUE
            ENDIF
   30    CONTINUE
C
C     FORM A NEW CLUSTER BY AMALGAMATING ROWS OF CLOSEST PAIR
C
         I=IMIN
         IWORK(1,K)=I
         II=IWORK(3,JMIN)
         WORK(K)=DMIN
         IWORK(2,K)=II
         IWORK(3,JMIN)=-IWORK(3,I)
         IWORK(3,I)=II
         CALL AMALG(N,A(IMIN,1),A(JMIN,1),MM)
   40 CONTINUE
C
C     REORDER OBJECTS, ASSIGN LABELS
C
      J=M+1
      KC=M-1
      IF(M.EQ.2) RETURN
      L=IWORK(2,KC)
      DO 50 JJ=1,M
         J=M-JJ+2
         LL=-IWORK(3,L)
         IWORK(3,L)=J
         L=LL
   50 CONTINUE
      DO 60 K=1,KC
         L=IWORK(1,K)
         IWORK(1,K)=IWORK(3,L)
         L=IWORK(2,K)
   60    IWORK(2,K)=IWORK(3,L)
      DO 70 I=1,M
         J=IWORK(3,I)
   70    TLAB(J)=RLAB(I)
C
C     DEFINE LAST TWO CLUSTERS, AND CLUSTER DIAMETERS
C
      DMAX=0.
      DO 80 I=1,KC
   80    IF(WORK(I).GT.DMAX) DMAX=WORK(I)
      IF (OUNIT .GT. 0) THEN
         WRITE(OUNIT,1)
    1    FORMAT('1')
         WRITE(OUNIT,2) DMAX
    2    FORMAT(' MAXIMUM JOINING DISTANCE =',F20.6)
      ENDIF
      DO 90 I=1,KC
   90    IWORK(3,I)=(WORK(I)/DMAX)*100
      DO 100 J=1,3
         DO 100 K=KC,M
  100       IWORK(J,K)=IWORK(J,KC)
C
C     PRODUCE OUTPUT
C
      TLAB(1) = TITLE
      CALL TREE1(M+1,M-1,DMIWRK,IWORK,TLAB,IERR,OUNIT)
      RETURN
      END
